# GitHub Receiver

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [development]: traces   |
|               | [alpha]: metrics   |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Areceiver%2Fgithub%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Areceiver%2Fgithub) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Areceiver%2Fgithub%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Areceiver%2Fgithub) |
| Code coverage | [![codecov](https://codecov.io/github/open-telemetry/opentelemetry-collector-contrib/graph/main/badge.svg?component=receiver_github)](https://app.codecov.io/gh/open-telemetry/opentelemetry-collector-contrib/tree/main/?components%5B0%5D=receiver_github&displayType=list) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@adrielp](https://www.github.com/adrielp), [@crobert-1](https://www.github.com/crobert-1), [@TylerHelmuth](https://www.github.com/TylerHelmuth) |

[development]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#development
[alpha]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#alpha
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

# Table of Contents

- [Overview](#overview)
- [Metrics - Getting Started](#metrics---getting-started)
  - [Scraping](#scraping)
- [Traces - Getting Started](#traces---getting-started)
  - [Receiver Configuration](#receiver-configuration)
  - [Configuring Service Name](#configuring-service-name)
  - [Configuration a GitHub App](#configuration-a-github-app)

## Overview

The GitHub receiver receives data from [GitHub](https://github.com) via two methods:

1. Scrapes [version control system][vcsm] metrics from GitHub repositories and
organizations using the GraphQL and REST APIs.
2. Receives GitHub Actions events by serving a webhook endpoint, converting
those events into traces.

## Metrics - Getting Started

The current default set of metrics can be found in
[documentation.md](./documentation.md).

These metrics can be used as leading indicators ([capabilities][doracap])
to the [DORA][dorafour] metrics; helping provide insight into modern-day
engineering practices.

The collection interval is common to all scrapers and is set to 30 seconds by default.

> Note: Generally speaking, if the vendor allows for anonymous API calls, then you
> won't have to configure any authentication, but you may only see public repositories
> and organizations. You may also run into significantly more rate limiting.

```yaml
github:
    collection_interval: <duration> #default = 30s recommended 300s
    scrapers:
        scraper/config-1:
        scraper/config-2:
        ...
```

A more complete example using the GitHub scrapers with authentication is as follows:

```yaml
extensions:
    bearertokenauth/github:
        token: ${env:GH_PAT}

receivers:
    github:
        initial_delay: 1s
        collection_interval: 60s
        scrapers:
            scraper:
                metrics: #Optional
                    vcs.contributor.count:
                        enabled: true
                github_org: <myfancyorg> 
                search_query: "org:<myfancyorg> topic:<o11yalltheway>" # Recommended optional query override, defaults to "{org,user}:<github_org>"
                endpoint: "https://selfmanagedenterpriseserver.com" # Optional
                auth:
                    authenticator: bearertokenauth/github
service:
    extensions: [bearertokenauth/github]
    pipelines:
        metrics:
            receivers: [..., github]
            processors: []
            exporters: [...]
```

### Configuration

`github_org` (**required**): Specify the GitHub organization or username to scrape.

`endpoint` (optional): Set this only when using a self-managed GitHub instance (e.g., `https://selfmanagedenterpriseserver.com` -- SHOULD NOT include `api` subdomain or `/graphql` context path).

`search_query` (optional): A filter to narrow down repositories. Defaults to `org:<github_org>` (or `user:<username>`). For example, use `repo:<org>/<repo>` to target a specific repository. Any valid GitHub search syntax is allowed.

`metrics` (optional): Enable or disable metrics scraping. See the [metrics documentation](./documentation.md) for details.

### Scraping

> Important:
> * The GitHub scraper does not emit metrics for branches that have not had
>   changes since creation from the default branch (trunk).
> * Due to GitHub API limitations, it is possible for the branch time metric to
>   change when rebases occur, recreating the commits with new timestamps.

For additional context on GitHub scraper limitations and inner workings please
see the [Scraping README][ghsread].

[ghsread]: internal/scraper/githubscraper/README.md#github-limitations


### GitHub Personal Access Token (PAT) Setup

To create a GitHub Personal Access Token (PAT), please refer to the official [documentation](https://docs.github.com/en/authentication/keeping-your-account-and-data-secure/managing-your-personal-access-tokens).

**Organization or Personal Access:**
When generating the PAT, select the appropriate `Resource owner` — either your personal account or the organization and choose the correct `Repository access` type. For fine-grained tokens, explicitly configure the necessary `Repository permissions` or `Organization permissions`.

**Note**: 
The PAT must have read access to the target repositories. If the PAT doesn't have permission to access repositories in the target organization, only the repository count metric will be available. Detailed repository metrics cannot be fetched.

## Traces - Getting Started

Workflow tracing support is accomplished through the processing of GitHub
Actions webhook events for workflows and jobs. The [`workflow_job`][wjob] and
[`workflow_run`][wrun] event payloads are then constructed into `trace`
telemetry.

Each GitHub Action workflow or job, along with its steps, are converted
into trace spans, allowing the observation of workflow execution times,
success, and failure rates. Each Trace and Span ID is deterministic. This
enables the underlying actions to emit telemetry from any command running in any
step. This can be achieved by using tools like the [run-with-telemetry
action][run] and [otel-cli][otcli]. The key is generating IDs in the same way
that this GitHub receiver does. The [trace_event_handling.go][tr] file contains
the `new*ID` functions that generate deterministic IDs.

**IMPORTANT** - Workflow Job names MUST be unique in each workflow for
deterministic span IDs to not conflict with eachother. GitHub does not enforce
this behavior, but when linting a workflow, warns that there are duplicate job
names.

### Receiver Configuration

**IMPORTANT** - Ensure your WebHook endpoint is secured with a secret and a Web
Application Firewall (WAF) or other security measure.

The WebHook configuration exposes the following settings:

* `endpoint`: (default = `localhost:8080`) - The address and port to bind the WebHook to.
* `path`: (default = `/events`) - The path for Action events to be sent to.
* `health_path`: (default = `/health`) - The path for health checks.
* `secret`: (optional) - The secret used to [validates the payload][valid].
* `required_header`: (optional) - The required header key and value for incoming requests.
* `service_name`: (optional) - The service name for the traces. See the
[Configuring Service Name](#configuring-service-name) section for more
information.

The WebHook configuration block also accepts all the [confighttp][cfghttp]
settings.

An example configuration is as follows:

```yaml
receivers:
    github:
        webhook:
            endpoint: localhost:19418
            path: /events
            health_path: /health
            secret: ${env:SECRET_STRING_VAR}
            required_headers:
                WAF-Header: "value"
```

For tracing, all configuration is set under the `webhook` key. The full set
of exposed configuration values can be found in [`config.go`](./config.go).

### Configuring Service Name

The `service_name` option in the WebHook configuration can be used to set a
pre-defined `service.name` resource attribute for all traces emitted by the
receiver. This takes priority over the internal generation of the
`service.name`. In this configuration, it would be important to create a GitHub
receiver per GitHub app configured for the set of repositories that match your
`service.name`.

However, a more efficient approach would be to leverage the default generation
of `service.name` by configuring [Custom Properties][cp] in each GitHub
repository. To do that simply add a `service_name` key with the desired value in
each repository and all events sent to the GitHub receiver will properly
associate with that `service.name`. Alternatively, the `service_name` will be
derived from the repository name.

The precedence for setting the `service.name` resource attribute is as follows:

1. `service_name` configuration in the WebHook configuration.
2. `service_name` key in the repository's Custom Properties per repository.
3. `service_name` derived from the repository name.
4. `service.name` set to `unknown_service` per the semantic conventions as a fall back.

### Configuring A GitHub App

To configure a GitHub App, you will need to create a new GitHub App within your
organization. Refer to the general [GitHub App documentation][ghapp] for how to
create a GitHub App. During the subscription phase, subscribe to `workflow_run` and `workflow_job` events.

[wjob]: https://docs.github.com/en/webhooks/webhook-events-and-payloads#workflow_job
[wrun]: https://docs.github.com/en/webhooks/webhook-events-and-payloads#workflow_run
[valid]: https://docs.github.com/en/webhooks/using-webhooks/validating-webhook-deliveries
[cfghttp]: https://pkg.go.dev/go.opentelemetry.io/collector/config/confighttp#ServerConfig
[cp]: https://docs.github.com/en/organizations/managing-organization-settings/managing-custom-properties-for-repositories-in-your-organization
[vcsm]: https://opentelemetry.io/docs/specs/semconv/cicd/cicd-metrics/#vcs-metrics
[doracap]: https://dora.dev/capabilities/
[dorafour]: https://dora.dev/guides/dora-metrics-four-keys/
[ghapp]: https://docs.github.com/en/apps/creating-github-apps/registering-a-github-app/registering-a-github-app
[run]: https://github.com/krzko/run-with-telemetry
[otcli]: https://github.com/equinix-labs/otel-cli
[tr]: ./trace_event_handling.go
