# K8slog Receiver

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [development]: logs   |
| Distributions | [] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Areceiver%2Fk8slog%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Areceiver%2Fk8slog) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Areceiver%2Fk8slog%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Areceiver%2Fk8slog) |
| Code coverage | [![codecov](https://codecov.io/github/open-telemetry/opentelemetry-collector-contrib/graph/main/badge.svg?component=receiver_k8slog)](https://app.codecov.io/gh/open-telemetry/opentelemetry-collector-contrib/tree/main/?components%5B0%5D=receiver_k8slog&displayType=list) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@h0cheung](https://www.github.com/h0cheung), [@TylerHelmuth](https://www.github.com/TylerHelmuth) |

[development]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#development
<!-- end autogenerated section -->

Tails and parses logs in k8s environment.

There only one mode of discovery as for now, it's specified by the `discovery.mode` configuration option:
- `daemonset-stdout`: (default) Deployed as a DaemonSet, the receiver will read logs from the stdout of pods in the same node.

Two modes of discovery are planned to be supported in the future:

- `daemonset-file`: Deployed as a DaemonSet, the receiver will read logs from files inside pods in the same node.
- `sidecar`: Deployed as a sidecar container, the receiver will read logs from files.

## Configuration

The following settings are common to all discovery modes:

| Field            | Default            | Description                                                                                                      |
|------------------|--------------------|------------------------------------------------------------------------------------------------------------------|
| `discovery.mode` | `daemonset-stdout` | The mode of discovery. Only `daemonset-stdout` is supported now. `daemonset-file` and `sidecar` are coming soon. |
| `extract`        |                    | The rules to extract metadata from pods and containers. TODO default values.                                     |
| TODO: add fields for reading files similar to filelogreceiver |

When `discovery.mode` is not `sidecar`, there are additional configuration options:

| Field                         | Default          | Description                                                                                                                                                   |
|-------------------------------|------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------|
| `discovery.k8s_api.auth_type` | `serviceAccount` | The authentication type of k8s api. Options are `serviceAccount` or `kubeConfig`.                                                                             |
| `discovery.host_root`         | `/host-root`     | The directory which the root of host is mounted on.                                                                                                           |
| `discovery.runtime_apis`      |                  | The runtime apis used to get log file paths. docker and cri-containerd are supported now. By default, it will try to automatically detect the cri-containerd. |
| `discovery.node_from_env`     | `KUBE_NODE_NAME` | The environment variable name of node name.                                                                                                                   |
| `discovery.filter`            | []               | The filter used to filter pods and containers. By default, all pods and containers will be collected.                                                         |

### Operators

Each operator performs a simple responsibility, such as parsing a timestamp or JSON. Chain together operators to process logs into a desired format.

- Every operator has a `type`.
- Every operator can be given a unique `id`. If you use the same type of operator more than once in a pipeline, you must specify an `id`. Otherwise, the `id` defaults to the value of `type`.
- Operators will output to the next operator in the pipeline. The last operator in the pipeline will emit from the receiver. Optionally, the `output` parameter can be used to specify the `id` of another operator to which logs will be passed directly.
- Only parsers and general purpose operators should be used.

### Filters

When `discovery.mode` is not `sidecar`, the `discovery.filter` field can be used to filter pods and containers. The filter is a list of rules. Each rule is a map with the following fields:

| Field         | Description                                                  |
|---------------|--------------------------------------------------------------|
| `annotations` | MapFilters that filters pods by annotations.                 |
| `labels`      | MapFilters that filters pods by labels.                      |
| `env`         | MapFilters that filters containers by environment variables. |
| `containers`  | ValueFilters that filters containers by name.                |
| `namespaces`  | ValueFilters that filters pods by namespace.                 |
| `pods`        | ValueFilters that filters pods by name.                      |

#### MapFilter

A MapFilter can be used to filter pods by maps, such as annotations or labels. It has the following fields:

| Field   | Description                                                                                                                                                                                                                                                                                                                                                                                                                               |
|---------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| `op`    | The operation to perform. Options are: <br> - "equals": (default) the value must be equal to the specified value. <br>- "not-equals": the value must not be equal to the specified value. <br> - "exists": the value must exist. <br> - "not-exists": the value must not exist. <br> - "matches": the value must match the specified regular expression. <br> - "not-matches": the value must not match the specified regular expression. |
| `key`   | The key of the map.                                                                                                                                                                                                                                                                                                                                                                                                                       |
| `value` | The value to match. Only used for "equals", "not-equals", "matches", and "not-matches" operations.                                                                                                                                                                                                                                                                                                                                        |

#### ValueFilter

A ValueFilter can be used to filter pods by string values, such as container names or namespaces. It has the following fields:

| Field   | Description                                                                                                                                                                                                                                                                                                                                         |
|---------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| `op`    | The operation to perform. Options are: <br> - "equals": (default) the value must be equal to the specified value. <br>- "not-equals": the value must not be equal to the specified value. <br> - "matches": the value must match the specified regular expression. <br> - "not-matches": the value must not match the specified regular expression. |
| `value` | The value to match.                                                                                                                                                                                                                                                                                                                                 |

### Extract

The `extract` field can be used to extract fields from the log file path. It has the following fields:

| Field         | Description                                                                          |
|---------------|--------------------------------------------------------------------------------------|
| `metadata`    | A string slice of metadata to extract from the pods and containers.                  |
| `env`         | A FieldExtractConfig that extracts fields from environment variables of containers.  |
| `annotations` | A FieldExtractConfig that extracts fields from annotations of pods.                  |
| `labels`      | A FieldExtractConfig that extracts fields from labels of pods.                       |

#### FieldExtractConfig

A FieldExtractConfig can be used to extract fields from maps, such as annotations or labels. It has the following fields:

| Field       | Description                                                                                          |
|-------------|------------------------------------------------------------------------------------------------------|
| `tag_name`  | Required. The name of the extracted attributes.                                                      |
| `key`       | The key of the map (annotation, label or etc).Exactly one of `key` or `key_regex` must be specified. |
| `key_regex` | The regular expression of the key. Exactly one of `key` or `key_regex` must be specified.            |
| `regex`     | Optional. The regular expression to extract a submatch from the value.                               |

### Supported encodings

| Key        | Description                                                      |
|------------|------------------------------------------------------------------|
| `nop`      | No encoding validation. Treats the file as a stream of raw bytes |
| `utf-8`    | UTF-8 encoding                                                   |
| `utf-16le` | UTF-16 encoding with little-endian byte order                    |
| `utf-16be` | UTF-16 encoding with big-endian byte order                       |
| `ascii`    | ASCII encoding                                                   |
| `big5`     | The Big5 Chinese character encoding                              |

Other less common encodings are supported on a best-effort basis. See [https://www.iana.org/assignments/character-sets/character-sets.xhtml](https://www.iana.org/assignments/character-sets/character-sets.xhtml) for other encodings available.

## Additional Terminology and Features

- An [entry](../../pkg/stanza/docs/types/entry.md) is the base representation of log data as it moves through a pipeline. All operators either create, modify, or consume entries.
- A [field](../../pkg/stanza/docs/types/field.md) is used to reference values in an entry.
- A common [expression](../../pkg/stanza/docs/types/expression.md) syntax is used in several operators. For example, expressions can be used to [filter](../../pkg/stanza/docs/operators/filter.md) or [route](../../pkg/stanza/docs/operators/router.md) entries.

### Parsers with Embedded Operations

Many parsers operators can be configured to embed certain followup operations such as timestamp and severity parsing. For more information, see [complex parsers](../../pkg/stanza/docs/types/parsers.md#complex-parsers).

## Example - Collect logs from stdout of all containers

Receiver Configuration
```yaml
receivers:
  k8slog:
    discovery:
      mode: daemonset-stdout
    operators:
      - type: recombine
        combine_field: body
        is_first_entry: body matches "^\\d{4}-\\d{2}-\\d{2}"
        max_log_size: 128kb
        source_identifier: attributes["k8s.pod.uid"]
```
